@@ -39,7 +39,7 @@ static void UserEditCb(GtkWidget *w, gpointer data)
 	Callsign = gtk_entry_get_text(GTK_ENTRY(UserCall));
 	Locator  = gtk_entry_get_text(GTK_ENTRY(UserLocator));
 
-	if (strlen(Locator) != 6) {
+	if (strlen(Locator) != 6 && strlen(Locator) != 8) {
 		Message_Box("The Locator is not the correct length");
 		return;
 	}
