@@ -55,6 +55,7 @@ static double X_Scale;
 static double Y_Scale;
 
 static void Draw_Map(void);
+static void Draw_Arrow(int, int, int);
 
 static int Normalise_X_Text(int x, char *text)
 {
@@ -148,7 +149,7 @@ static void Draw_Map(void)
 	char Buffer[101];
 	double last_lat, last_long;
 	double llat, llong;
-	double last_x = 0.0, last_y = 0.0;
+	int last_x = 0, last_y = 0;
 	int x = 0, y = 0;
 	short *n, *slat, *slong;
 	FILE *fp;
@@ -266,7 +267,7 @@ static void Draw_Beacons(void)
 {
 	struct Beacon_Struct *Beacon_Data;
 	GSList *Beacon;
-	int x, y;
+	int i, x, y;
 
 	Beacon = Beacon_List;
 
@@ -278,7 +279,10 @@ static void Draw_Beacons(void)
 			y = Lat_To_Y(Beacon_Data->Latitude);
 
 			if (X_In_Range(x) && Y_In_Range(y)) {
-				gdk_draw_rectangle(MapData, DetailBeaconsGC, TRUE, x - 2, y - 2, 4, 4);
+				gdk_draw_rectangle(MapData, DetailBeaconsGC, TRUE, x - 3, y - 3, 6, 6);
+
+				for (i = 0; i < Beacon_Data->Angle_Count; i++)
+					Draw_Arrow(x, y, Beacon_Data->Angle[i]);
 
 				x = Normalise_X_Text(x, Beacon_Data->Callsign);
 				y = Normalise_Y_Text(y, Beacon_Data->Callsign);
@@ -291,6 +295,40 @@ static void Draw_Beacons(void)
 	}
 }
 
+static void Draw_Arrow(int x, int y, int angle)
+{
+	int line_end_x, line_end_y;
+	int back_arrow_x, back_arrow_y;
+	double sin_angle, cos_angle;
+	int x_len, y_len;
+	GdkPoint points[3];
+
+	sin_angle = sin(RAD((double)angle));
+	cos_angle = cos(RAD((double)angle));
+
+	line_end_x = x + (int)(14.0 * sin_angle + 0.5);
+	line_end_y = y - (int)(14.0 * cos_angle + 0.5);
+
+	gdk_draw_line(MapData, DetailBeaconsGC, x, y, line_end_x, line_end_y);
+
+	back_arrow_x = x + (int)(7.0 * sin_angle + 0.5);
+	back_arrow_y = y - (int)(7.0 * cos_angle + 0.5);
+
+	x_len = (int)(3.0 * cos_angle + 0.5);
+	y_len = (int)(3.0 * sin_angle + 0.5);
+
+	points[0].x = back_arrow_x - x_len;
+	points[0].y = back_arrow_y - y_len;
+
+	points[1].x = line_end_x;
+	points[1].y = line_end_y;
+
+	points[2].x = back_arrow_x + x_len;
+	points[2].y = back_arrow_y + y_len;
+
+	gdk_draw_polygon(MapData, DetailBeaconsGC, TRUE, points, 3);
+}
+
 static gint DetailMapDestroyCb(GtkWidget *w, gpointer data)
 {
 	gtk_widget_destroy(DetailWindow);
@@ -338,6 +376,7 @@ static gint DetailMapMotionCb(GtkWidget *widget, GdkEventMotion *event)
 	double Bearing, Distance;
 	double Best_Distance;
 	char buf[51], buffer[101];
+	int i;
 
 	Latitude  = Y_To_Lat(event->y);
 	Longitude = X_To_Long(event->x);
@@ -393,13 +432,22 @@ static gint DetailMapMotionCb(GtkWidget *widget, GdkEventMotion *event)
 			buffer[0] = '\0';
 
 			if (strlen(Best_Beacon->Antenna) > 0) {
-				sprintf(buf, "Ant: %-15s", Best_Beacon->Antenna);
+				sprintf(buf, "Ant: %s", Best_Beacon->Antenna);
 				strcat(buffer, buf);
 			}
 
-			if (strlen(Best_Beacon->Direction) > 0) {
-				sprintf(buf, " - %-10s", Best_Beacon->Direction);
-				strcat(buffer, buf);
+			if (Best_Beacon->Angle_Count == 0) {
+				strcat(buffer, " - Omni");
+			} else {
+				strcat(buffer, " - ");
+
+				for (i = 0; i < Best_Beacon->Angle_Count; i++) {
+					sprintf(buf, "%d", Best_Beacon->Angle[i]);
+					if (i > 0) strcat(buffer, "/");
+					strcat(buffer, buf);
+				}
+
+				strcat(buffer, " deg");
 			}
 
 			if (strlen(buffer) > 0)
@@ -473,7 +521,7 @@ void Draw_Detail_Map(double Latitude, double Longitude)
 {
 	GtkWidget *vbox;
 	GtkWidget *separator;
-	char Locator[7];
+	char Locator[9];
 	char buffer[51];
 
 	if (!Convert_Angles(Latitude, Longitude, Locator)) {
