@@ -34,7 +34,7 @@
 
 void Read_City_Data(void)
 {
-	char Buffer[120], *t[5];
+	char Buffer[120], *t[2];
 	char File_Name[81], *s;
 	struct City_Struct Temp_Data, *City;
 	FILE *fp;
@@ -89,7 +89,7 @@ static gint Beacon_Comparison(gconstpointer d1, gconstpointer d2)
 
 void Read_Beacon_Data(int type, char *filename)
 {
-	char Buffer[255], File_Name[81], *s, *t[5];
+	char Buffer[255], File_Name[81], *s, *p, *t[2];
 	double Latitude, Longitude;
 	struct Beacon_Struct Temp_Data, *Beacon = NULL;
 	int valid = FALSE;
@@ -146,13 +146,20 @@ void Read_Beacon_Data(int type, char *filename)
 			} else if (strcmp(t[0], "antenna") == 0) {
 				strcpy(Temp_Data.Antenna, t[1]);
 			} else if (strcmp(t[0], "direction") == 0) {
-				strcpy(Temp_Data.Direction, t[1]);
+				s = t[1];
+				while ((p = strtok(s, ",")) != NULL) {
+					Temp_Data.Angle[Temp_Data.Angle_Count] = atoi(p);
+					Temp_Data.Angle_Count++;
+					s = NULL;
+				}
 			} else if (strcmp(t[0], "height") == 0) {
 				Temp_Data.Height = atoi(t[1]);
 			} else if (strcmp(t[0], "power") == 0) {
 				strcpy(Temp_Data.Power, t[1]);
 			} else if (strcmp(t[0], "operator") == 0) {
 				strcpy(Temp_Data.Operator, t[1]);
+			} else if (strcmp(t[0], "mode") == 0) {
+				/* ignore */
 			} else {
 				fprintf(stderr, "baken: unknown keyword %s in %s\n", t[0], File_Name);
 			}
