@@ -79,6 +79,10 @@
 #define	TYPE_MISC	9
 #define	MAX_TYPES	10
 
+#define	LIST_FREQUENCY		0
+#define	LIST_DISTANCE		1
+#define	LIST_BEARING		2
+
 #define	COLOUR_BACKGROUND	0
 #define	COLOUR_BEACONS		1
 #define	COLOUR_CITIES		2
@@ -100,23 +104,28 @@
 #define	SQUARE_HEIGHT	(FIELD_HEIGHT / 10.0)	/* 0 - 9 */
 #define	SUB_WIDTH	(SQUARE_WIDTH / 24.0)	/* A - X */
 #define	SUB_HEIGHT	(SQUARE_HEIGHT / 24.0)	/* A - X */
+#define	SUB_SUB_WIDTH	(SUB_WIDTH / 10.0)	/* 0 - 9 */
+#define	SUB_SUB_HEIGHT	(SUB_HEIGHT / 10.0)	/* 0 - 9 */
 
 #ifndef PI
 #define	PI		3.141592654
 #endif
 
-#define	R		6378.16
+/* International ellipsoid of 1924 */
+#define	RA		6378.388		/* Semi-major axis */
+#define	RB		6356.912		/* Semi-minor axis */
 
 struct Beacon_Struct {
 	int    Type;
 	char   Callsign[21];
 	double Frequency;
-	char   Locator[7];
+	char   Locator[10];
 	double Latitude;
 	double Longitude;
 	char   QTH[51];
 	char   Antenna[51];
-	char   Direction[51];
+	int    Angle[10];
+	int    Angle_Count;
 	int    Height;
 	char   Power[51];
 	char   Operator[21];
@@ -124,7 +133,7 @@ struct Beacon_Struct {
 
 struct User_Struct {
 	char   Callsign[21];
-	char   Locator[7];
+	char   Locator[10];
 	double Latitude;
 	double Longitude;
 	int    Bands[MAX_BANDS];
@@ -235,6 +244,10 @@ extern gint MainMapClickCb(GtkWidget *, GdkEventButton *);
 /* In Utils.C								*/
 /************************************************************************/
 
+extern double RAD(double);
+
+extern double DEG(double);
+
 extern int Convert_Locator(char *, double *, double *);
 
 extern int  Convert_Angles(double, double, char *);
