@@ -24,6 +24,51 @@
 
 #include "global.h"
 
+struct Beacon_Plus {
+	struct Beacon_Struct *Data;
+	double                Bearing;
+	double                Distance;
+};
+
+static gint Frequency_Comparison(gconstpointer a, gconstpointer b)
+{
+	struct Beacon_Plus *beacon1 = (struct Beacon_Plus *)a;
+	struct Beacon_Plus *beacon2 = (struct Beacon_Plus *)b;
+
+	 if (beacon1->Data->Frequency > beacon2->Data->Frequency)
+	 	return 1;
+	 else if (beacon1->Data->Frequency < beacon2->Data->Frequency)
+	 	return -1;
+	 else
+	 	return 0;
+}
+
+static gint Bearing_Comparison(gconstpointer a, gconstpointer b)
+{
+	struct Beacon_Plus *beacon1 = (struct Beacon_Plus *)a;
+	struct Beacon_Plus *beacon2 = (struct Beacon_Plus *)b;
+
+	 if (beacon1->Bearing > beacon2->Bearing)
+	 	return 1;
+	 else if (beacon1->Bearing < beacon2->Bearing)
+	 	return -1;
+	 else
+	 	return 0;
+}
+
+static gint Distance_Comparison(gconstpointer a, gconstpointer b)
+{
+	struct Beacon_Plus *beacon1 = (struct Beacon_Plus *)a;
+	struct Beacon_Plus *beacon2 = (struct Beacon_Plus *)b;
+
+	 if (beacon1->Distance > beacon2->Distance)
+	 	return 1;
+	 else if (beacon1->Distance < beacon2->Distance)
+	 	return -1;
+	 else
+	 	return 0;
+}
+
 void ListCb(GtkWidget *w, gpointer data)
 {
 	GtkWidget *window;
@@ -38,17 +83,21 @@ void ListCb(GtkWidget *w, gpointer data)
 	char DistBuffer[21];
 	char *title[7];
 	char *items[7];
-	GSList *Beacon;
+	GSList *Beacon, *Sorted_Beacon_List = NULL;
 	struct Beacon_Struct *Data;
+	int sort_order;
+	struct Beacon_Plus *Data_Plus;
 
 	if (Beacon_List == NULL) {
-		Warning_Box("No Beacons loaded !");
+		Warning_Box("No Data loaded !");
 		return;
 	}
 
+	sort_order = (int)data;
+
 	window = gtk_window_new(GTK_WINDOW_DIALOG);
 
-	gtk_window_set_title(GTK_WINDOW(window), "Beacon Bearings and Distances");
+	gtk_window_set_title(GTK_WINDOW(window), "Frequencies, Bearings and Distances");
 	gtk_container_set_border_width(GTK_CONTAINER(window), BORDER_WIDTH);
 
 	vbox = gtk_vbox_new(FALSE, BORDER_WIDTH);
@@ -98,58 +147,91 @@ void ListCb(GtkWidget *w, gpointer data)
 		if (Check_Beacon(Data)) {
 			Calc_Distance_Bearing(User_Data.Latitude, User_Data.Longitude, Data->Latitude, Data->Longitude, &Bearing, &Distance);
 
-			sprintf(FreqBuffer, "%.3f", Data->Frequency);
-			items[0] = FreqBuffer;
+			Data_Plus = g_malloc(sizeof(struct Beacon_Plus));
 
-			switch (Data->Type) {
-				case TYPE_IARU:
-					items[1] = "IARU";
+			Data_Plus->Data     = Data;
+			Data_Plus->Bearing  = Bearing;
+			Data_Plus->Distance = Distance;
+
+			switch (sort_order) {
+				case LIST_FREQUENCY:
+					Sorted_Beacon_List = g_slist_insert_sorted(Sorted_Beacon_List, Data_Plus, Frequency_Comparison);
 					break;
-				case TYPE_NONIARU:
-					items[1] = "Non-IARU";
+				case LIST_BEARING:
+					Sorted_Beacon_List = g_slist_insert_sorted(Sorted_Beacon_List, Data_Plus, Bearing_Comparison);
 					break;
-				case TYPE_REPEATER:
-					items[1] = "Repeater";
-					break;
-				case TYPE_LINEAR:
-					items[1] = "Linear";
-					break;
-				case TYPE_MAILBOX:
-					items[1] = "Mailbox";
-					break;
-				case TYPE_DXCLUSTER:
-					items[1] = "DX Cluster";
-					break;
-				case TYPE_NODE:
-					items[1] = "Node";
-					break;
-				case TYPE_RS:
-					items[1] = "Rain-Scatter";
-					break;
-				case TYPE_MISC:
-					items[1] = "Misc";
+				case LIST_DISTANCE:
+					Sorted_Beacon_List = g_slist_insert_sorted(Sorted_Beacon_List, Data_Plus, Distance_Comparison);
 					break;
 				default:
-					items[1] = "Unknown";
+					g_error("Unknown value for list sorting");
 					break;
 			}
-
-			items[2] = Data->Callsign;
-
-			items[3] = Data->Locator;
-
-			sprintf(BearBuffer, "%.0f", Bearing);
-			items[4] = BearBuffer;
-
-			sprintf(DistBuffer, "%.0f", Distance);
-			items[5] = DistBuffer;
-
-			gtk_clist_append(GTK_CLIST(listlist), items);
 		}
 
 		Beacon = g_slist_next(Beacon);
 	}
 
+	Beacon = Sorted_Beacon_List;
+
+	while (Beacon != NULL) {
+		Data_Plus = (struct Beacon_Plus *)Beacon->data;
+
+		sprintf(FreqBuffer, "%.3f", Data_Plus->Data->Frequency);
+		items[0] = FreqBuffer;
+
+		switch (Data_Plus->Data->Type) {
+			case TYPE_IARU:
+				items[1] = "IARU";
+				break;
+			case TYPE_NONIARU:
+				items[1] = "Non-IARU";
+				break;
+			case TYPE_REPEATER:
+				items[1] = "Repeater";
+				break;
+			case TYPE_LINEAR:
+				items[1] = "Linear";
+				break;
+			case TYPE_MAILBOX:
+				items[1] = "Mailbox";
+				break;
+			case TYPE_DXCLUSTER:
+				items[1] = "DX Cluster";
+				break;
+			case TYPE_NODE:
+				items[1] = "Node";
+				break;
+			case TYPE_RS:
+				items[1] = "Rain-Scatter";
+				break;
+			case TYPE_MISC:
+				items[1] = "Misc";
+				break;
+			default:
+				items[1] = "Unknown";
+				break;
+		}
+
+		items[2] = Data_Plus->Data->Callsign;
+
+		items[3] = Data_Plus->Data->Locator;
+
+		sprintf(BearBuffer, "%.0f", Data_Plus->Bearing);
+		items[4] = BearBuffer;
+
+		sprintf(DistBuffer, "%.0f", Data_Plus->Distance);
+		items[5] = DistBuffer;
+
+		gtk_clist_append(GTK_CLIST(listlist), items);
+
+		g_free(Data_Plus);
+
+		Beacon = g_slist_next(Beacon);
+	}
+
+	g_slist_free(Sorted_Beacon_List);
+
 	gtk_clist_thaw(GTK_CLIST(listlist));
 
 	gtk_widget_show(button);
